@@ -24,6 +24,6 @@ del get_distribution, DistributionNotFound
 
 __author__ = 'Christoph Wehmeyer'
 __email__ = 'christoph.wehmeyer@fu-berlin.de'
-__credits__ = ['Guillermo Pérez-Hernández'],
+__credits__ = ['Guillermo Pérez-Hernández', 'Martin K. Scherer'],
 
 from .mdshare import fetch, load, catalogue, search
